module STypes where

{- Module Types was derived by hand from the XML specification, following
   the grammar precisely.  This module, STypes, is a simplification of
   Types, removing layers of indirection and redundancy, and generally
   making things easier to work with.
-}

data Document = Document Prolog Element
data Prolog   = Prolog (Maybe XMLDecl) (Maybe DocTypeDecl)
data XMLDecl  = XMLDecl VersionInfo (Maybe EncodingDecl) (Maybe SDDecl) 
data Misc     = Comment Comment
              | PI ProcessingInstruction
              
type ProcessingInstruction = (PITarget,String)

type SDDecl      = Bool 
type VersionInfo = String 
type Comment     = String 
type PITarget    = String 

data DocTypeDecl = DTD Name (Maybe ExternalID) [MarkupDecl] 
data MarkupDecl  = Element  ElementDecl
                 | AttList  AttListDecl
                 | Entity   EntityDecl
                 | Notation NotationDecl
                 | MarkupMisc Misc
                 | MarkupPE PEReference MarkupDecl

data ExtSubset     = ExtSubset (Maybe TextDecl) [ExtSubsetDecl] 
data ExtSubsetDecl = ExtMarkupDecl MarkupDecl
                   | ExtConditionalSect ConditionalSect
                   | ExtPEReference PEReference ExtSubsetDecl

data Element   = Elem Name [Attribute] [Content]
data ElemTag   = ElemTag Name [Attribute]	-- intermediate for parsing
type Attribute = (Name, AttValue)
data Content   = CElem Element
               | CString CharData
               | CRef Reference
               | CMisc Misc

data ElementDecl = ElementDecl Name ContentSpec 
data ContentSpec = EMPTY
                 | ANY
                 | Mixed Mixed
                 | ContentSpec CP
                 | ContentPE PEReference ContentSpec
data CP = TagName Name Modifier
        | Choice [CP] Modifier
        | Seq [CP] Modifier 
        | CPPE PEReference CP
data Modifier = None  --JustOne
              | Query --ZeroOrOne
              | Star  --ZeroOrMore
              | Plus  --OneOrMore 
data Mixed = PCDATA
           | PCDATAplus [Name] 

data AttListDecl = AttListDecl Name [AttDef] 
data AttDef      = AttDef Name AttType DefaultDecl 
data AttType     = StringType
                 | TokenizedType TokenizedType
                 | EnumeratedType EnumeratedType 
data TokenizedType = ID
                   | IDREF
                   | IDREFS
                   | ENTITY
                   | ENTITIES
                   | NMTOKEN
                   | NMTOKENS 
data EnumeratedType = NotationType NotationType
                    | Enumeration Enumeration 
type NotationType   = [Name]	-- nonempty list
type Enumeration    = [NmToken]	-- nonempty list
data DefaultDecl    = REQUIRED
                    | IMPLIED
                    | Default AttValue (Maybe FIXED) 
data FIXED          = FIXED 

data ConditionalSect = IncludeSect IncludeSect
                     | IgnoreSect IgnoreSect 
type IncludeSect = ExtSubsetDecl
type IgnoreSect  = [IgnoreSectContents]
newtype Ignore   = Ignore String 
data IgnoreSectContents = IgnoreSectContents Ignore [(IgnoreSectContents,Ignore)] 

data Reference    = RefEntity EntityRef
                  | RefChar CharRef 
                  deriving Eq
newtype EntityRef = EntityRef Name 
                  deriving Eq
newtype CharRef   = CharRef String 
                  deriving Eq
type PEReference  = Name 

data EntityDecl   = EntityGEDecl GEDecl
                  | EntityPEDecl PEDecl 
data GEDecl       = GEDecl Name EntityDef 
data PEDecl       = PEDecl Name PEDef 
data EntityDef    = DefEntityValue EntityValue
                  | DefExternalID ExternalID (Maybe NDataDecl) 
data PEDef        = PEDefEntityValue EntityValue
                  | PEDefExternalID ExternalID 
data ExternalID   = SYSTEM SystemLiteral
                  | PUBLIC PubidLiteral SystemLiteral 
newtype NDataDecl = NDATA Name  

data TextDecl     = TextDecl (Maybe VersionInfo) EncodingDecl 
data ExtParsedEnt = ExtParsedEnt (Maybe TextDecl) Content 
data ExtPE        = ExtPE (Maybe TextDecl) ExtSubsetDecl 

data NotationDecl    = NOTATION Name (Either ExternalID PublicID) 
newtype PublicID     = PUBLICID PubidLiteral 
newtype EncodingDecl = EncodingDecl String 

type Name     = String		 -- non-empty string
type Names    = [Name]		 -- non-empty list
type NmToken  = String		 -- non-empty string
type NmTokens = [NmToken]	 -- non-empty list

data AttValue    = AttValue [Either String Reference]
                 deriving Eq
data EntityValue = EntityValue [EV] 
data EV = EVString String
        | EVPERef PEReference EV
        | EVRef Reference 
newtype PubidLiteral  = PubidLiteral String 
newtype SystemLiteral = SystemLiteral String 
type CharData         = String 
type CDSect           = CharData

instance Eq ElemTag where
    (ElemTag n _) == (ElemTag m _)  = n==m
